;
; AutoIt Version: 3.0
; Language:       English
; Platform:       Win9x/NT
; Author:         Michael W. Bayley         
;
; Script Function:
;   Export Bookmarks as MSIE Favorites

#include <Constants.au3>
#include <GUIConstants.au3>

#include "TrayUrlDefs.au3"
#include "TrayUrlIniFunc.au3"

;
; Global values
;

; Title Information
$TitleBar = "TrayURL MSIE Favorites Exporter Version 2.0, (c) 2007 Michael W. Bayley"
$TitleBar2 = "TrayURL MSIE Favorites Exporter"

$Tag = ""
$Major = 0
$Minor = 0

; URL Entry Values
$Categories = 0
$Items = 0
$Index = 0
$Index2 = 0
$NotDone = 0
$Cat = ""
$Cat2 = ""
$Record = CreateRecord("", "", "", "", 0, "")
$Name = ""
$Url = ""
$Text = ""

; MSIE Favorites Values
$FavoritesFolder = ""
$TrayUrlFolder = "TrayURL"
$FavoritesExt = ".url"
$FavoritesSection = "InternetShortcut"
$FavoritesKey = "URL"

; Dialog Values
$TextSelect1 = "You can elect to export your bookmarks to Microsoft Internet Explorer "
$TextSelect1 = $TextSelect1 & "Favorites to either " & Chr(34) & "All Users" & Chr(34)
$TextSelect1 = $TextSelect1 & " or to the current user.  If you select the "
$TextSelect1 = $TextSelect1 & "custom option, then you can select the base "
$TextSelect1 = $TextSelect1 & "folder to export your bookmarks to.  Please select "
$TextSelect1 = $TextSelect1 & "which option to import from."
$TextUser = "Current User:"
$TextCommon = "All Users"
$TextUser = "Current User"
$TextCustom = "Custom"
$TextCancel = "Cancel"

$TextCat1 = "FAVORITES FOLDER ALREADY EXISTS!"
$TextCat2 = "Current name:"
$TextCat3 = "New folder name:"
$TextCat4 = "Do you wish to rename the folder or append the bookmarks to the existing Favorites folder?"
$TextRename = "Rename"
$TextAppend = "Append"

$TextUrl1 = "FAVORITE ALREADY EXISTS!"
$TextUrl2 = "Favorite folder:"
$TextUrl3 = "Current name:"
$TextUrl4 = "New name:"
$TextUrl5 = "Do you wish to rename the Favorite or overwrite the existing Favorite?"
$TextOverwrite = "Overwrite"

;
; Main Code
;
; Entry:       <None>
; Return:      0 - No errors
;              1 - Fatal error encountered
; Procedure:   1. If command line parameter then message & exit.
;              3. Error if ini file not found.
;              3. Verify it is a TrayURL profile, error if not.
;              4. Get user to pick User Favorite or Common Favorites.
;              5. Export the bookmarks.
;              6. Exit all done.
;

; 1. If command line parameter then message & exit.

$msg = "This program will export the TrayURL categories & URLs to the Microsoft Internet Explorer" & @CRLF
$msg = $msg & " Favorites folder.  You will have the choice of exporting to the common Favorites folder for"  & @CRLF
$msg = $msg & "all users or the Favorites folder of the current user. If you run the program with no command" & @CRLF
$msg = $msg & "line arguments then the default profile will be used, otherwise you can specify a single" & @CRLF
$msg = $msg & "profile on the command line to export from it." & @CRLF & @CRLF
$msg = $msg & "In all instances the profile file msut exist in the same directory as the program." & @CRLF & @CRLF
MsgBox(0, $TitleBar, $msg)

If $CmdLine[0] > 1 Then
   MsgBox(0, $TitleBar, "Error - Illegal number of command line arguments!")
   Exit(1)
ElseIf $CmdLine[0] == 1 Then
   $IniFile = @ScriptDir
   If StringRight($IniFile, 1) == "\" Then
      $IniFile = $IniFile & $CmdLine[1] & ".ini"
   Else
      $IniFile = $IniFile & "\" & $CmdLine[1] & ".ini"
   EndIf
EndIf


; 2. Error if ini file not found.
If FileExists($IniFile) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Profile " & $IniFile & " not found!")
   Exit(1)   
EndIf


; 3. Verify it is a TrayURL profile, error if not.
$Tag = IniRead($IniFile, $IniProgram, $IniTag, "")
$Major = Number(IniRead($IniFile, $IniProgram, $IniMajor, 0))
$Minor = Number(IniRead($IniFile, $IniProgram, $IniMinor, 0))

If $Tag <> $ProgramID Then
   MsgBox(0, $TitleBar, "Error - Profile " & $IniFile & " does not appear to be valid!")
   Exit(1)   
EndIf

If $Major <> $ProgramMajor OR $Minor <> $ProgramMinor Then
   $msg = "WARNING! The profile appears to be from a different version of TrayURL." & @CRLF
   $msg = $msg & "Proceeding may produce unknown results." & @CRLF & @CRLF
   $msg = $msg & "Proceed anyway?" & @CRLF & @CRLF
   If MsgBox(260, $TitleBar, $msg) <> 6 Then Exit(0)
EndIf


; 4. Get user to pick User Favorite or Common Favorites.
$FavoritesFolder = FavoritesDialog()
If StringLen($FavoritesFolder) == 0 Then Exit(0)


; 5. Export the bookmarks.
; Pick export to folder.
$msg = "Please select a Favorites base folder to export to (leave blank "
$msg &= "to export to the root folder):"
$TrayUrlFolder = InputBox($TitleBar2, $msg, $TrayUrlFolder, "", 300, 140)
If @error Then Exit(0) 

; Process Main category
$Cat = $IniCatMain
$Items = IniRead($IniFile, $Cat, $IniItems, 0)
If $Items Then
   $NotDone = 1
   $Cat2 = $TrayUrlFolder
   While $NotDone
      $CatFolder = DirAppend($FavoritesFolder, $Cat2)
      If FileExists($CatFolder) Then
         If StringLen($Cat2) Then
            $Cat2 = CategoryDialog($Cat2)
         EndIf
         If StringLen($Cat2) == 0 Then
            $NotDone = 0
         EndIf
      Else
         $NotDone = 0
      EndIf
   WEnd
      
   ; Create the category folder
   If StringLen($Cat2) Then
      $FavoritesFolder = DirAppend($FavoritesFolder, $Cat2)
      If DirCreate($FavoritesFolder) <> 1 Then
         MsgBox(0, $TitleBar, "Error - Unable to create folder " & $FavoritesFolder & "!  Aborting!")
         Exit(1)
      EndIf
   Else
      $FavoritesFolder = DirAppend($FavoritesFolder, $TrayUrlFolder)
   EndIf
   
   ; Process entries
   For $index2 = 1 To $Items
      ; Read next item
      $Record = ReadRecordByIndex($TitleBar2, $Cat, $Index2)
   
      ; Write it out if not blank
      If StringLen($Record[$RECNAME]) AND StringLen($Record[$RECURL]) Then
         If FileExists(DirAppend($FavoritesFolder, $Record[$RECNAME] & $FavoritesExt)) Then
            $Record[$RECNAME] = NameDialog($FavoritesFolder, $Record[$RECNAME])
         EndIf
         
         WriteFavorite($FavoritesFolder, $Record[$RECNAME] & $FavoritesExt, $Record[$RECURL])
      EndIf
   Next
EndIf 

; Process the categories
$Categories = IniRead($IniFile, $IniCatSection, $IniItems, 0)
$Index = 1
While $Index <= $Categories
   $Cat = IniRead($IniFile, $IniCatSection, $IniItem & $Index, "")
   If StringLen($Cat) Then
      $Items = IniRead($IniFile, $Cat, $IniItems, 0)
      If $Items Then
         $NotDone = 1
         $Cat2 = $Cat
         While $NotDone
            $CatFolder = DirAppend($FavoritesFolder, $Cat2)
            If FileExists($CatFolder) Then
               $Cat2 = CategoryDialog($Cat2)
               If StringLen($Cat2) == 0 Then
                  $NotDone = 0
               EndIf
            Else
               $NotDone = 0
            EndIf
         WEnd
         
         ; Create the category folder
         If StringLen($Cat2) Then
            If DirCreate($CatFolder) <> 1 Then
               MsgBox(0, $TitleBar, "Error - Unable to create folder " & $CatFolder & "!  Aborting!")
               Exit(1)
            EndIf
         EndIf
   
         ; Process entries
         For $index2 = 1 To $Items
            ; Read next item
            $Record = ReadRecordByIndex($TitleBar, $Cat, $Index2)
   
            ; Write it out if not blank
            If StringLen($Record[$RECNAME]) AND StringLen($Record[$RECURL]) Then
               If FileExists(DirAppend($CatFolder, $Record[$RECNAME] & $FavoritesExt)) Then
                  $Record[$RECNAME] = NameDialog($CatFolder, $Record[$RECNAME])
               EndIf

               WriteFavorite($CatFolder, $Record[$RECNAME] & $FavoritesExt, $Record[$RECURL])
            EndIf
         Next
      EndIf
   EndIf
   $Index = $Index + 1
WEnd


; 6. Exit all done.
MsgBox(0, $TitleBar, "Process Complete!")
Exit(0)


;
; User Defined Functions
; Dialog Functions
;

;
; FavoritesDialog()
; Inputs:         <None>
; Returns:        Path to Favorites folder to use
;                 NULL if user aborts
; Procedure:      1. Get Common Favorites.
;                 2. Get User Favorites. 
;                 3. Display dialog.
;                 4, Get user response.
;                 5. Return data.
;
Func  FavoritesDialog()
   Local    $dlg
   Local    $edit1, $button1, $button2, $button3, $button4
   Local    $notDone, $notDone2, $msg
   Local    $common, $user

   ; 1. Get Common Favorites.
   $common = @FavoritesCommonDir


   ; 2. Get User Favorites.
   $user = @FavoritesDir 


   ; 3. Display dialog.
   $dlg = GUICreate($TitleBar2, 370, 194)
   GUICtrlCreateLabel($TextSelect1, 25, 12, 320, 70)
   GUICtrlCreateLabel($TextUser, 98, 100, 80, 20)
   $edit1 = GUICtrlCreateInput(@UserName, 174, 98, 100, 20)
   $button1 = GUICtrlCreateButton($TextCommon, 10, 142, 80, 30)
   $button2 = GUICtrlCreateButton($TextUser, 100, 142, 80, 30)
   $button3 = GUICtrlCreateButton($TextCustom, 190, 142, 80, 30)
   $button4 = GUICtrlCreateButton($TextCancel, 280, 142, 80, 30)
   GUICtrlSetState($edit1, $GUI_DISABLE)
   GUISetState(@SW_SHOW, $dlg)


   ; 4. Get user input.
   $notDone = 1
   While $notDone
      $msg = GUIGetMsg($dlg)
      Select
         Case $msg == $GUI_EVENT_CLOSE
            $msg = ""
            $notDone = 0
         Case $msg == $button1
            $msg = $common
            $notDone = 0
         Case $msg == $button2
            $msg = $user
            $notDone = 0
         Case $msg == $button3
            GUISetState(@SW_DISABLE, $dlg)
            If StringInStr(@UserProfileDir, "\Profiles\") == 0 Then
               $msg = @WindowsDir
            Else
               $msg = @UserProfileDir
               $msg = StringLeft($msg, StringInStr($msg, "\", 1, -1) - 1)
               If StringLen($msg) ==2 AND StringRight($msg, 1) == ":" Then
                  $msg = $msg & "\"
               EndIf
            EndIf
            $msg = FileSelectFolder($TitleBar2, "", 0, $msg)
            GUISetState(@SW_ENABLE, $dlg)
            GUISetState(@SW_HIDE, $dlg)
            GUISetState(@SW_SHOW, $dlg)
            If StringLen($msg) Then $notDone = 0
         Case $msg == $button4
            $msg = ""
            $notDone = 0
      EndSelect
   WEnd


   ; 5. Return data.
   GUIDelete($dlg)
   Return($msg)
EndFunc


;
; CategoryDialog($cat)
; Inputs:         $cat     Category in conflict
; Returns:        Path to folder to use
;                 NULL if use same folder
; Procedure:      1. Display dialog
;                 2. Get user input. 
;                 3. Return the data.
;
Func  CategoryDialog($cat)
   Local    $dlg
   Local    $edit1, $edit2, $button1, $button2
   Local    $notDone, $msg, $newcat

   ; 1. Display dialog.
   $dlg = GUICreate($TitleBar2, 360, 218)
   GUICtrlCreateLabel($TextCat1, 70, 16, 260, 20)
   GUICtrlCreateLabel($TextCat2, 40, 50, 100, 20)
   $edit1 = GUICtrlCreateInput($cat, 135, 48, 180, 20)
   GUICtrlCreateLabel($TextCat3, 40, 80, 100, 20)
   $edit2 = GUICtrlCreateInput($cat, 135, 78, 180, 20)
   GUICtrlCreateLabel($TextCat4, 30, 120, 300, 40)
   $button1 = GUICtrlCreateButton($TextRename, 100, 168, 80, 30)
   $button2 = GUICtrlCreateButton($TextAppend, 200, 168, 80, 30)
   GUICtrlSetState($edit1, $GUI_DISABLE)
   GUICtrlSetState($button1, $GUI_DEFBUTTON)
   GUISetState(@SW_SHOW, $dlg)


   ; 2. Get user input.
   $notDone = 1
   While $notDone
      $msg = GUIGetMsg($dlg)
      Select
         Case $msg == $button1
            $newcat = GUICtrlRead($edit2)
            $notDone = 0
         Case $msg == $button2
            $newcat = ""
            $notDone = 0
      EndSelect
   WEnd


   ; 3. Return the data.
   GUIDelete($dlg)
   Return($newcat)
EndFunc


;
; NameDialog($folder, $name)
; Inputs:         $folder  Folder containg URL in conflict
;                 $name    URL name in conflict
; Returns:        New name for Favorite file to use
;                 $name for overwrite
; Procedure:      1. Display dialog
;                 2. Get user input. 
;                 3. Return the data.
;
Func  NameDialog($folder, $name)
   Local    $dlg
   Local    $edit1, $edit2, $edit3, $button1, $button2
   Local    $notDone, $msg, $newname

   ; 1. Display dialog.
   $dlg = GUICreate($TitleBar2, 360, 248)
   GUICtrlCreateLabel($TextUrl1, 108, 16, 260, 20)
   GUICtrlCreateLabel($TextUrl2, 30, 50, 100, 20)
   $edit1 = GUICtrlCreateInput($folder, 120, 48, 210, 20)
   GUICtrlCreateLabel($TextUrl3, 30, 80, 100, 20)
   $edit2 = GUICtrlCreateInput($name, 120, 78, 210, 20)
   GUICtrlCreateLabel($TextUrl4, 30, 110, 100, 20)
   $edit3 = GUICtrlCreateInput($name, 120, 108, 210, 20)
   GUICtrlCreateLabel($TextUrl5, 30, 150, 310, 40)
   $button1 = GUICtrlCreateButton($TextRename, 90, 198, 80, 30)
   $button2 = GUICtrlCreateButton($TextOverwrite, 190, 198, 80, 30)
   GUICtrlSetState($edit1, $GUI_DISABLE)
   GUICtrlSetState($edit2, $GUI_DISABLE)
   GUICtrlSetState($button1, $GUI_DEFBUTTON)
   GUISetState(@SW_SHOW, $dlg)


   ; 2. Get user input.
   $notDone = 1
   While $notDone
      $msg = GUIGetMsg($dlg)
      Select
         Case $msg == $button1
            $newname = GUICtrlRead($edit3)
            $notDone = 0
         Case $msg == $button2
            $newname = $name
            $notDone = 0
      EndSelect
   WEnd


   ; 3. Return data.
   GUIDelete($dlg)
   Return($newname)
EndFunc


;
; WriteFavorite($folder, $name, $url)
; Inputs:         $folder  Path to favorite folder
;                 $name    Filename of URL
;                 $url     URL to write
; Returns:        0        All ok
;                 1        Write failure
; Procedure:      1. Create the file, error if fail.
;                 2. Write the URL, report if error.
;                 3. Exit all done.
;
Func  WriteFavorite($folder, $name, $url)
      Local    $handle

      ; 1. Create the file, report if fail.
      $handle = FileOpen(DirAppend($folder, $name), 2)
      If $handle == -1 Then
         MsgBox(0, $TitleBar, "Error - Failure creating Favorite " & Chr(34) & $name & Chr(34) & " in category folder " & $cat & "!")
         Return(1)
      EndIf
      FileClose($handle)
      
      
      ; 2. Write the URL, report if failure.
      If IniWrite(DirAppend($folder, $name), $FavoritesSection, $FavoritesKey, $url) <> 1 Then
         MsgBox(0, $TitleBar, "Error - Failure writing Favorite " & Chr(34) & $name & Chr(34) & " in category folder " & $cat & "!")
         Return(1)
      EndIf
   
   
   ; 3. Rexit all done.
   Return(0)
EndFunc


;
; User Defined Functions
; Miscelaneous Functions
;

;
; DirAppend($dir, $file)
;
; Description:    Append the file string to the directory string.  Insert
;                 a "\" if needed.
; Inputs:         $dir     Directory string to append to
;                 $file    File string to append
; Procedure:      1. If directory is null then return file.
;                 2. If directory ends in "\" then $dir + $file.
;                 3. Else $dir + "\" + $file.
;
Func  DirAppend($dir, $file)
   ; 1. If directory is null then return file.
   If StringLen($dir) == 0 Then
      Return($file)
   EndIf

   ; 2. If directory ends in "\" then $dir + $file.
   If StringRight($dir, 1) == "\" Then
      Return($dir & $file)
   EndIf
   
   ; 3. Else $dir + "\" + $file.
   Return($dir & "\" & $file)
EndFunc

