;
;
; AutoIt Version: 3.0
; Language:       English
; Platform:       Win9x/NT
; Author:         Michael W. Bayley         
;
; Script Function:
;   TrayUrlPaster - Paste User Login & Pasword into Program

#include <Constants.au3>
#include <GUIConstants.au3>

#include "TrayUrlDefs.au3"
#include "TrayUrlIniDefs.au3"

;
; Global values
;

; Title Information
$TitleBar = "TrayURL Paster Version 2.0, (c) 2007 Michael W. Bayley"
$TitleBar2 = "TrayURL Paster"

$Tag = ""
$Major = 0
$Minor = 0

; Paster Values
$LoginHotKey = ""
$PaswdHotKey = ""
$ExitHotKey = ""
$Login = ""
$Paswd = ""
$AutoTabs = 0
$AutoCR = 0
$AutoClose = 0

; Dialog Values
$Dlg = -1
$Button1 = -1
$Button2 = -1
$Button3 = -1
$TextLogin = "Login"
$TextPaswd = "Password"
$DlgActive = -1


;
; Main Code
;
; Entry:       <None>
; Return:      0 - No errors
;              1 - Fatal error encountered
; Procedure:   1. If command line parameter incorrect then message & exit.
;              3. Error if ini file not found.
;              3. Verify it is a TrayURL profile, error if not.
;              4. Read in Paster settings.
;              5. Setup HotKeys.
;              6. Display dialog.
;              7. Process dialog functions.
;              8. Exit all done.
;

; 1. If command line parameter then message & exit.
AutoItSetOption("SendKeyDelay", 2)

If $CmdLine[0] <> 4 Then
   $msg = "This program is a companion program for TrayURL.  It will allow you to paste your Login" & @CRLF
   $msg &= "Password for a website into a webpage using either hotkeys or buttons on a small dialog"  & @CRLF
   $msg &= "box."  & @CRLF & @CRLF
   $msg &= "The command line syntax is: TrayUrlPaster.exe <ProfileFile> <Login> <Password> <flags>" & @CRLF & @CRLF
   $msg &= "Under normal operation, the program is run by TrayURL and the command line parameters" & @CRLF
   $msg &= "are handled by the program." & @CRLF
   MsgBox(0, $TitleBar, $msg)
   Exit(1)
EndIf

If StringInStr($CmdLine[1], ":") == 2 Then
   $IniFile = $CmdLine[1]
ElseIf StringLeft($CmdLine[1], 1) == "\" Then
   $IniFile = StringLeft(@ScriptDir, 2) & $CmdLine[1]
Else
   $IniFile = @ScriptDir
   If StringRight($IniFile, 1) == "\" Then
      $IniFile = $IniFile & $CmdLine[1]
   Else
      $IniFile = $IniFile & "\" & $CmdLine[1]
   EndIf
EndIf


; 2. Error if ini file not found.
If FileExists($IniFile) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Profile " & $IniFile & " not found!")
   Exit(1)   
EndIf


; 3. Verify it is a TrayURL profile, error if not.
$Tag = IniRead($IniFile, $IniProgram, $IniTag, "")
$Major = Number(IniRead($IniFile, $IniProgram, $IniMajor, 0))
$Minor = Number(IniRead($IniFile, $IniProgram, $IniMinor, 0))

If $Tag <> $ProgramID Then
   MsgBox(0, $TitleBar, "Error - Profile " & $IniFile & " does not appear to be valid!")
   Exit(1)   
EndIf

If $Major <> $ProgramMajor OR $Minor <> $ProgramMinor Then
   MsgBox(0, $TitleBar, "Error - Incompatible version of configuration file!")
   Exit(1)
EndIf


; 4. Read in Paster settings.
$LoginHotKey = IniRead($IniFile, $IniPasterSection, $IniLoginHotKey, "")
$PaswdHotKey = IniRead($IniFile, $IniPasterSection, $IniPaswdHotKey, "")
$ExitHotKey = IniRead($IniFile, $IniPasterSection, $IniExitHotKey, "")
$Login = $CmdLine[2]
$Paswd = $CmdLine[3]
$AutoTabs = BitAND(Number($CmdLine[4]), $TabsMask)
$AutoCR = BitAND(Number($CmdLine[4]), $CrFlag)
$AutoClose = BitAND(Number($CmdLine[4]), $CloseFlag)


; 5. Setup HotKeys.
If StringLen($LoginHotKey) Then HotKeySet($LoginHotKey , "PasteLogin")
If StringLen($PaswdHotKey) Then HotKeySet($PaswdHotKey , "PastePassword")
If StringLen($ExitHotKey) Then HotKeySet($ExitHotKey , "ActivateExit")


; 6. Display dialog.
$Dlg = GUICreate($TitleBar2, 123, 65, 20, 20, $WS_CAPTION, $WS_EX_TOPMOST)
$TextClose = "X"
$Button1 = GUICtrlCreateButton($TextClose, 8, 7, 18, 50)
$Button2 = GUICtrlCreateButton($TextLogin, 35, 7, 80, 20)
$Button3 = GUICtrlCreateButton($TextPaswd, 35, 37, 80, 20)
GUICtrlSetState($Button1, $GUI_DEFBUTTON)
GUISetState(@SW_SHOW, $dlg)


; 7. Process Dialog functions.
While $DlgActive
   $msg = GUIGetMsg()

   Select
      Case $msg == $Button1
            ; Close dialog
            ExitLoop
      Case $msg == $Button2
            ; Copy login (and more?)
            Send("!{TAB}")
            Send($Login)
            If $AutoTabs Then
               $Count = $AutoTabs
               While $Count
                  Send("{TAB}")
                  $Count -= 1
               WEnd
               Send($Paswd)
               If $AutoCR Then Send("{ENTER}")
               If $AutoClose Then $DlgActive = 0
            EndIf
      Case $msg == $Button3
            ; Copy password (and more?)
            Send("!{TAB}")
            Send($Paswd)
            If $AutoCR Then Send("{ENTER}")
            If $AutoClose Then $DlgActive = 0
      Case $msg == $Button1
            ; Close dialog
            ExitLoop
   EndSelect
WEnd


; 8. Exit all done.
GUIDelete($Dlg)
Exit(0)


;
; User Defined Functions
; HotKey Fuctions
;

;
; PasteLogin() - Paste login into application
; Entry:          <None>
; Return:         <None>
; Procedure:      1. Paste login (and more?) into application.
;                 2. Exit all done.
;
Func  PasteLogin()
   Local    $count

   ; 1. Paste login (and more?) into application.
   Send($Login)
   If $AutoTabs Then
      $Count = $AutoTabs
      While $Count
         Send("{TAB}")
         $count -= 1
      WEnd
      Send($paswd)
      If $AutoCR Then Send("{ENTER}")
      If $AutoClose Then $DlgActive = 0
   EndIf


   ; 2. Exit all done.
   Return(0)
EndFunc


;
; PastePassword() - Paste password into application
; Entry:          <None>
; Return:         <None>
; Procedure:      1. Paste password (and more?) into application.
;                 2. Exit all done.
;
Func  PastePassword()
   ; 1. Paste password into application.
   Send($Paswd)
   If $AutoCR Then Send("{ENTER}")
   If $AutoClose Then $DlgActive = 0


   ; 2. Exit all done.
   Return(0)
EndFunc


;
; ActivateExit() - Force the program to terminate
; Entry:          <None>
; Return:         <None>
; Procedure:      1. Flag the exit.
;                 2. Exit all done.
;
Func  ActivateExit()
   ; 1. Flag the exit.
   $DlgActive = 0

   ; 2. Exit all done.
   Return(0)
EndFunc

